# coding: utf-8

require 'chef/knife/hitori_base'
require 'knife-hitori/lib/interactive_configure'

class Chef
  class Knife
    class HitoriConfig < Knife

      banner 'knife hitori config (options)'

      include HitoriBase
      include KnifeHitori::InteractiveConfigure
      CHEF_DIR = '.chef'
      KNIFE_FILE = 'knife.rb'
      EC2_FILE = 'ec2.rb'

      option :base_dir,
             :short => '-b COOKBOOK_ROOT_DIR',
             :long => '--base COOKBOOK_ROOT_DIR',
             :description => 'Chef ROOT Dir of cookbooks, data_bags...',
             :default => Dir::getwd

      deps do
        require 'erubis'
      end

      def run
        create_knife_ruby
        conf = ask_ec2_config_all(Chef::Config.knife)
        conf[:template_file] = KnifeHitori::resource('centos_bootstrap.erb')
        conf[:ssh_port] = 22
        create_knife_ec2_ruby(conf)
      end

      def base_dir
        config[:base_dir]
      end

      def chef_dir
        "#{base_dir}/#{CHEF_DIR}"
      end

      def create_knife_ruby
        knife_ruby = "#{chef_dir}/#{KNIFE_FILE}"
        FileUtils.makedirs(chef_dir) unless ::Dir.exists?(chef_dir)
        if ::File.exists?(knife_ruby)
          yes_no = input_arg("overwrite knife.rb (#{knife_ruby}) [y/n] ?", 'N') {|x| x =~ /^[yn]$/i}
          if yes_no.downcase != 'y'
            ui.info ui.color('cancel overwrite knife.rb', :yellow)
            return
          end
        end

        template = KnifeHitori::resource('knife.erb')
        erubis = Erubis::Eruby.new(::File.read(template))
        ::File.write(knife_ruby, erubis.result({}))
        ui.info ui.color("write knife.rb in #{knife_ruby}", :green)
      end

      def create_knife_ec2_ruby(conf)
        knife_ec2_ruby = "#{chef_dir}/#{EC2_FILE}"
        template = KnifeHitori::resource('knife_ec2.erb')
        erubis = Erubis::Eruby.new(::File.read(template))
        ::File.write(knife_ec2_ruby, erubis.result(config: conf))
        ui.info ui.color("save settings in #{knife_ec2_ruby}", :green)
      end
    end
  end
end