# coding: utf-8

require 'chef/knife/hitori_base'
require 'set'

require 'knife-hitori/lib/run_remote_cook'
require 'chef/knife/hitori/run_list_options'

class Chef
  class Knife
    class HitoriCook < Knife
      banner 'knife hitori cook (options)'
      include HitoriBase
      include RunListOptions

      deps do
        require 'fog'
      end

      option :install_chef,
             :long => '--install-chef',
             :description => 'Install chef before run cookbook. This process usually have done by hitori-prepare.'

      #########
      option :tag,
             :short => '-t TAG_KEY=TAG_VALUE',
             :long  => '--tag TAG_KEY=TAG_VALUE',
             :description => 'specify servers by EC2 Tag'

      option :groups,
             :short => '-g GROUP1[,GROUP2]',
             :long  => '--groups GROUP1[,GROUP2]',
             :description => 'specify servers by Security Group'

      option :name,
             :short => '-n NAME',
             :long  => '--name NAME',
             :description => 'specify a server by Instance Name (Name Tag)'

      option :public_ip_address,
             :long  => '--ip IP_ADDRESS',
             :description => 'specify a server by Public IP Address'

      def run
        update_environment(config[:environment]) if config[:environment]
        run_remote
      end

      ## @return [Hash]
      #def json_attrs
      #  attrs = load_settings || {}
      #  # Chef::Mixin::DeepMerge.merge(normal_attrs,normal_attrs_to_merge)
      #
      #  # Update run_list
      #  run_list = new_run_list
      #  unless run_list.empty?
      #    attrs['run_list'] = run_list
      #  end
      #  #
      #  attrs
      #end
      #
      #def load_settings
      #  path = Chef::Config[:settings_path]
      #  if ::File.exists?(path)
      #    return JSON.parse(::File.read(path))
      #  end
      #  ui.warn "#{path} is not found!!" if path
      #  {}
      #end

      def new_run_list
        run_list = []
        run_list += config[:roles].split(/,/).map{|r| "role[#{r.strip}]"} if config[:roles]
        run_list += config[:recipes].split(/,/).map{|r| "recipe[#{r.strip}]"} if config[:recipes]
        run_list
      end

      #########################################################
      ########################################################
      def run_remote(server_list=nil)
        if server_list
          servers = server_list
        else
          servers = find_ec2_servers
          unless servers && !servers.empty?
            ui.warn ui.color('no server found', :red)
            return
          end
        end

        run_list = new_run_list
        threads = []
        servers.each_with_index do |server, idx|
          threads << Thread.new {
            KnifeHitori::RunRemoteCook.new(Chef::Config, config, server, :idx => idx, :ui => ui, :run_list => run_list).run
          }
        end
        threads.each do |t|
          t.join
        end
      end

      def find_ec2_servers
        server = nil
        3.times do
          server = fetch_server_info
          break if server && !server.empty?
          print '.'
          sleep 5
        end
        puts
        server
      end

      def fetch_server_info
        servers = server_info_list

        if config[:tag]
          tag_key, tag_value = config[:tag].to_s.split(/=/, 2)
          servers.select {|x| x.tags[tag_key] == tag_value.to_s}
        elsif config[:groups]
          g = Set.new(config[:groups].split(/,/))
          servers.select {|x| (Set.new(x.groups) & g).size > 0}
        elsif config[:name]
          servers.select {|x| x.tags['Name'] == config[:name]}
        elsif config[:public_ip_address]
          servers.select {|x| x.public_ip_address == config[:public_ip_address]}
        end
      end

      def server_info_list
        connection = Fog::Compute.new(
            :provider => 'AWS',
            :aws_access_key_id => Chef::Config[:knife][:aws_access_key_id],
            :aws_secret_access_key => Chef::Config[:knife][:aws_secret_access_key],
            :region => Chef::Config[:knife][:region]
        )
        connection.servers.all.to_a
      end
    end
  end
end
