# coding: utf-8

require 'chef/knife/hitori_base'

class Chef
  class Knife
    class HitoriDataBagEnc < Knife
      banner 'knife hitori data bag enc BAG ITEM -j JSON_FILE (options)'
      include HitoriBase

      option :json_file,
             :short => '-j JSON_FILE',
             :description => 'A json file for encryption'

      option :secret_file,
             :long => '--secret-file SECRET_FILE',
             :description => 'A file containing the secret key to use to encrypt data bag item values'

      def run
        update_environment(config[:environment]) if config[:environment]
        config[:bag], config[:item] = @name_args
        exit 1 unless validate

        create_data_bag
      end

      def create_data_bag
        data_bag_path = Chef::Config[:data_bag_path]
        secret = Chef::EncryptedDataBagItem.load_secret(secret_file_path)
        data = JSON.parse(File.read(config[:json_file]))
        exit 1 unless check_data(data)
        encrypted_data = Chef::EncryptedDataBagItem.encrypt_data_bag_item(data, secret)
        bag_dir = "#{data_bag_path}/#{config[:bag]}"
        FileUtils.mkpath(bag_dir)
        write_path = "#{bag_dir}/#{config[:item]}.json"
        File.write(write_path, encrypted_data.to_json)
        ui.info ui.color("Created encrypted data bag item at #{write_path}", :green)
      end

      def check_data(data)
        unless data['id'] == config[:item]
          ui.error ui.color(%Q|ITEM=#{config[:item]} must have {"id": "#{config[:item]}"}, but #{config[:json_file]} does not.|)
          return false
        end
        return true
      end

      def secret_file_path
        config[:secret_file] || Chef::Config[:encrypted_data_bag_secret]
      end

      def validate
        if config[:bag].nil? || config[:item].nil?
          ui.error('Please specify BAG and ITEM')
          return false
        end

        if config[:json_file].nil?
          ui.error('You have not provided a json file for encryption')
          return false
        end

        unless secret_file_path
          ui.error('Please specify EncryptKey by Chef Config "encrypted_data_bag_secret" or --secret-file')
          return false
        end

        return true
      end
    end
  end
end

