# coding: utf-8

require 'chef/knife/hitori_base'
require 'knife-hitori/lib/crypt_util'

class Chef
  class Knife
    class HitoriFileEnc < Knife
      banner 'knife hitori file enc -i IN_FILE -o OUT_FILE (options)'
      include HitoriBase
      include KnifeHitori::CryptUtil

      option :in_file,
             :short => '-i IN_FILE',
             :description => 'A file for encryption'

      option :out_file,
             :short => '-o OUT_FILE',
             :description => 'A file for store encrypted file'

      option :secret_file,
             :long => '--secret-file SECRET_FILE',
             :description => 'A file containing the secret key to use to encrypt data bag item values'

      def run
        update_environment(config[:environment]) if config[:environment]
        exit 1 unless validate

        secret = Chef::EncryptedDataBagItem.load_secret(secret_file_path)
        data = File.read(config[:in_file])
        File.write(config[:out_file], JSON.dump(encrypt(data, secret)))
        ui.info ui.color("Saved in #{config[:out_file]}", :green)
      end

      def secret_file_path
        config[:secret_file] || Chef::Config[:encrypted_data_bag_secret]
      end

      def validate
        unless config[:in_file]
          ui.error '-i IN_FILE must be specified'
          return false
        end

        unless ::File.exists?(config[:in_file])
          ui.error "IN_FILE: #{config[:in_file]} not found"
          return false
        end

        unless config[:out_file]
          ui.error '-o OUT_FILE must be specified'
          return false
        end

        unless ::File.exists?(secret_file_path)
          ui.error 'No secret key file is found. Please "knife hitori key create" first.'
          return false
        end
        return true
      end
    end
  end
end

